﻿using System.Collections.Generic;
using System.Linq;

namespace Eingabeverwaltung.Metamodell
{
	public class ComposedTree : Tree
	{
		/// <summary>
		/// OperationName is the name of the operation.
		/// </summary>
		public readonly Operation operation;
		public readonly List<Tree> children;		

		/// <summary>
		/// Creates a ComposedTree. Base gets operation.name as name and operation.sorts.Last() as sort.
		/// </summary>
		/// <param name="operation"></param>
		/// <param name="children"></param>
		public ComposedTree(Operation operation, List<Tree> children) : base(operation.name, operation.sorts.Last())
		{
			this.operation = operation;
			this.children  = children;
		}
		        
		public override string ToString()
        {
			string res = "";
			if (operation != null) res += operation.name;
			if (children.Count > 0) res += "( ";
			bool wasInLoop = false;
			foreach (Tree child in children)
			{
				res += child.ToString() + ",";
				wasInLoop = true;
			}
			if (wasInLoop) res = res.Remove(res.Length-1);
			if (children.Count > 0) res += " )";
			return res;
        }

		public override List<Tree> findSubTree(Tree tree)
		{
			List<Tree> result = new List<Tree>();
			if (Equals(tree))
            {
				result.Add(this);
				return result;
            }
			foreach (Tree child in children)
            {
				List<Tree> found = child.findSubTree(tree);
				if (found.Count > 0) result.AddRange(found);
			}
			return result;
		}

        public override List<Variable> findUsedVars()
        {
			List<Variable> result = new List<Variable>();
			foreach(Tree child in children)
				result.AddRange(child.findUsedVars());
			return result;
        }

        public override bool Equals(Tree tree)
        {
			bool result = false;
			ComposedTree cTree = tree as ComposedTree;
			if (cTree != null && cTree.operation == operation)
			{
				result = true;
				for (int i = 0; i < children.Count; i++)
				{
					result = result && children[i].Equals(cTree.children[i]);
				}
			}
			return result;
		}

        public override Tree Copy()
        {
			List<Tree> copiedChildren = new List<Tree>();
			// Not a foreach because the order is important and its safer when order is defined explicitly.
			for (int i = 0; i < children.Count; i++) 
				copiedChildren.Add(children[i].Copy());
			return new ComposedTree(operation, copiedChildren);
        }
    }
}
