﻿using Eingabeverwaltung.Metamodell;
using System;
using System.Collections.Generic;
using System.Collections.Immutable;
using Ueberpruefung;

namespace Testverwaltung
{
    public class InductionTest : Test
    {
        private readonly ImmutableList<string> testIDs;
        private readonly ImmutableList<string> shouldValues;
        private readonly ImmutableList<string> testIDsStruk;
        private readonly ImmutableList<string> shouldValuesStruk;

        public InductionTest() : base() 
        {
            testIDs = new List<string>()
            {
                "1,2",
                "30,37",
                "31,38",
                "32,40,43",
                "33d,41,43",
                "33u",
                "34c",
                "34cIS,43",
                "34h",
                "35",
                "36",
                "39"
            }.ToImmutableList();
            List<string> shouldValues = new List<string>();
            string L1 = "Lemma L1 geht ein in Teilaufgaben: L1 \r\n";
            string Lo = "Lemma L1 ohne Bewertung \r\n";
            string hb = "Hauptbeweis\r\n";
            string m32(string c) => aus.ErrorMessage(new List<Error>() { new Error(32, c+" ") }, true, "", false, -1, -1);
            foreach (string testID in testIDs)
            {
                string str;
                switch (testID)
                {
                    case "1,2"   : str = L1 + aus.ErrorMessage(new List<Error>() { new Error(1), new Error(2) }, true, "Fall null", true, -1, -1);  break;                        
                    case "30,37" : str = hb + aus.ErrorMessage(new List<Error>() { new Error(21), new Error(37,"y ") }, true, "Fall null", true, 1, -1)
                                       + Lo + aus.ErrorMessage(new List<Error>() { new Error(22) }, false, -1, -1)
                                            + aus.ErrorMessage(new List<Error>() { new Error(30) },true,"", false, -1, -1); ; break;
                    case "31,38" : str = L1 + aus.ErrorMessage(new List<Error>() { new Error(38), new Error(31) }, true, "", false, -1, -1); break;
                    case "32,40,43" 
                                 : str = hb + m32("inc")
                                            + aus.ErrorMessage(new List<Error>() { new Error(40, "m "), new Error(43) }, true, "IH", false, -1, -1)
                                       + L1 + aus.ErrorMessage(new List<Error>() { new Error(8) },true,"Fall null", true, 1, -1); break;
                    case "33d,41,43"
                                 : str = hb + m32("inc")
                                            + aus.ErrorMessage(new List<Error>() { new Error(43) }, true, "IH", false, -1, -1)
                                            + aus.ErrorMessage(new List<Error>() { new Error(41), new Error(33) }, true, "Fall null", false, -1, -1); break;
                    case "33u"   : str = L1 + m32("null")
                                            + aus.ErrorMessage(new List<Error>() { new Error(33) }, true, "Fall plus( null,plus( null,null ) ) = plus( null,null )", false, -1, -1); break;
                    case "34c"   : str = hb + m32("null")
                                            + aus.ErrorMessage(new List<Error>() { new Error(34), new Error(33) }, true, "Fall plus( null,null ) = null", false, -1, -1); break;
                    case "34cIS,43" 
                                 : str = hb + aus.ErrorMessage(new List<Error>() { new Error(43)}, true, "IH", false, -1, -1)
                                            + aus.ErrorMessage(new List<Error>() { new Error(34, ": fixed Y-0 : Nat : forall m : Nat : plus( inc( Y-0 ),m ) = plus( m,inc( Y-0 ) )") }, true, "Fall inc", false, -1, -1); break;
                    case "34h"   : str = hb + aus.ErrorMessage(new List<Error>() { new Error(34, ": fixed k : Nat : forall m : Nat : plus( k,m ) = plus( m,k )") }, true, "IH", false, -1, -1)
                                            + aus.ErrorMessage(new List<Error>() { new Error(9, "[ plus( m,k ) / y ]") }, true, "Fall inc", true, 2, -1); break;
                    case "35"    : str = hb + aus.ErrorMessage(new List<Error>() { new Error(35, "k ") }, true, "IH", false, -1, -1)
                                            + aus.ErrorMessage(new List<Error>() { new Error(35, "k ") }, true, "Fall inc", false, -1, -1); break;
                    case "36"    : str = hb + aus.ErrorMessage(new List<Error>() { new Error(36), new Error(37,"k ") }, true, "Fall null", true, 1, -1); break;
                    case "39"    : str = L1 + aus.ErrorMessage(new List<Error>() { new Error(39) }, true, -1, -1)
                                            + aus.ErrorMessage(new List<Error>() { new Error(21, "{ L1 }") }, true, 1, -1); break;
                    default: 
                        Console.WriteLine("Found an unexpected testID " + testID+". This test may not be tested correctly.");
                        str = aus.NoErrors(); ; break;
                }
                shouldValues.Add(str);
            }
            this.shouldValues = shouldValues.ToImmutableList();
            testIDsStruk = new List<string>() 
            { 
                "22",
                "42",
                "43"
            }.ToImmutableList();
            List<string> shouldValuesStruk = new List<string>();
            foreach (string testID in testIDsStruk)
            {
                string str;
                switch (testID)
                {
                    case "22": str = "Lemma Komm ohne Bewertung \r\n" 
                                   + aus.ErrorMessage(new List<Error>() { new Error(22) }, false, "", false, -1, -1); break;
                    case "42": str = hb + aus.ErrorMessage(new List<Error>() { new Error(42) }, true, "IH", false, -1, -1); break;
                    case "43": str = hb + aus.ErrorMessage(new List<Error>() { new Error(43) }, true, "IH3", false, -1, -1); break;
                    default:
                        Console.WriteLine("Found an unexpected testID " + testID + ". This test may not be tested correctly.");
                        str = aus.NoErrors(); ; break;
                }
                shouldValuesStruk.Add(str);
            }
            this.shouldValuesStruk = shouldValuesStruk.ToImmutableList();
            numTests = testIDs.Count + testIDsStruk.Count; 
        }
        public override bool run()
        {
            string path = "Tests\\ind-simpl\\";
            resultString = "Testing Induction\r\n\r\n";
            ein.Eingabe_Interface("ADT", path + "ADT_Bsp2.adt");
            ein.Eingabe_Interface("Task", path + "Task_Bsp2.task");
            ein.Eingabe_Interface("Proof", path + "Proof_correct.proof");
            Ueberpruefer.ueberpruefe(ein.GetContainer());
            resultString += assert(0, aus.NoErrors(), aus.printAllErrors(ein.GetContainer()));
            for(int i=0; i< testIDs.Count; i++)
            {
                ein.Eingabe_Interface("Proof", path + "Proof_error_"+testIDs[i]+".proof");
                Ueberpruefer.ueberpruefe(ein.GetContainer());
                resultString += assert(testIDs[i], shouldValues[i], aus.printAllErrors(ein.GetContainer()));
            }
            path = "Tests\\ind-struk\\";
            resultString += "\r\nTesting Induction - struktural extension\r\n\r\n";
            ein.Eingabe_Interface("ADT", path + "ADT_Bsp.adt");
            ein.Eingabe_Interface("Task", path + "Task_Bsp.task");
            ein.Eingabe_Interface("Proof", path + "Proof_correct.proof");
            Ueberpruefer.ueberpruefe(ein.GetContainer());
            resultString += assert(0, aus.NoErrors(), aus.printAllErrors(ein.GetContainer()));
            for (int i = 0; i < testIDsStruk.Count; i++)
            {
                ein.Eingabe_Interface("Proof", path + "Proof_error_" + testIDsStruk[i] + ".proof");
                Ueberpruefer.ueberpruefe(ein.GetContainer());
                resultString += assert(testIDsStruk[i], shouldValuesStruk[i], aus.printAllErrors(ein.GetContainer()));
            }
            return passed;
        }


    }
}
